/*
 * File metadata abstractions.
 *
 * Copyright (C) 2019, 2022, 2024 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <sys/stat.h>

#include <systypes/base.h>
#include <systypes/timespec.h>

EXTERN_C_BEGIN

/* File metadata abstraction. */

typedef struct
{
  sys_dev_t           st_dev;
  sys_ino_t           st_ino;
  sys_mode_t          st_mode;
  sys_nlink_t         st_nlink;
  sys_uid_t           st_uid;
  sys_gid_t           st_gid;
  sys_dev_t           st_rdev;
  sys_off_t           st_size;
  struct sys_timespec st_atim;
  struct sys_timespec st_mtim;
  struct sys_timespec st_ctim;
  sys_blksize_t       st_blksize;
  sys_blkcnt_t        st_blocks;

} sys_stat_t;

/* Macros to be used in different C library environments. */

#define COPY_STAT(s1, s2) \
( \
  s1->st_dev = s2->st_dev, \
  s1->st_ino = s2->st_ino, \
  s1->st_mode = s2->st_mode, \
  s1->st_nlink = s2->st_nlink, \
  s1->st_uid = s2->st_uid, \
  s1->st_gid = s2->st_gid, \
  s1->st_rdev = s2->st_rdev, \
  s1->st_size = s2->st_size, \
  s1->st_atim.tv_sec = s2->st_atim.tv_sec, \
  s1->st_atim.tv_nsec = s2->st_atim.tv_nsec, \
  s1->st_mtim.tv_sec = s2->st_mtim.tv_sec, \
  s1->st_mtim.tv_nsec = s2->st_mtim.tv_nsec, \
  s1->st_ctim.tv_sec = s2->st_ctim.tv_sec, \
  s1->st_ctim.tv_nsec = s2->st_ctim.tv_nsec, \
  s1->st_blksize = s2->st_blksize, \
  s1->st_blocks = s2->st_blocks \
);

/* Functions to be implemented in different C library environments. */

void systypes_copy_from_sys_stat(struct stat *st, sys_stat_t *sst);

void systypes_copy_to_sys_stat(struct stat *st, sys_stat_t *sst);

EXTERN_C_END

/* vim: tabstop=4 expandtab shiftwidth=4
*/
